library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity axi_reader_64 is
  generic (
    BUF_WIDTH : integer := 64
  );
  port (
    clk : in std_logic;
    reset : in std_logic;
    fifo_dout : out std_logic_vector(64-1 downto 0);
    fifo_we : out std_logic;
    fifo_wclk : out std_logic;
    fifo_length : in std_logic_vector(32-1 downto 0);
    fifo_full : in std_logic;
    S_AXI_ARADDR : out std_logic_vector(32-1 downto 0);
    S_AXI_ARLEN : out std_logic_vector(8-1 downto 0);
    S_AXI_ARVALID : out std_logic;
    S_AXI_ARREADY : in std_logic;
    S_AXI_ARSIZE : out std_logic_vector(3-1 downto 0);
    S_AXI_ARBURST : out std_logic_vector(2-1 downto 0);
    S_AXI_ARCACHE : out std_logic_vector(4-1 downto 0);
    S_AXI_ARPROT : out std_logic_vector(3-1 downto 0);
    S_AXI_RDATA : in std_logic_vector(64-1 downto 0);
    S_AXI_RRESP : in std_logic_vector(2-1 downto 0);
    S_AXI_RLAST : in std_logic;
    S_AXI_RVALID : in std_logic;
    S_AXI_RREADY : out std_logic;
    request : in std_logic;
    busy : out std_logic;
    addr : in std_logic_vector(32-1 downto 0);
    len : in std_logic_vector(8-1 downto 0)
  );
end axi_reader_64;

architecture RTL of axi_reader_64 is

  attribute mark_debug : string;
  attribute keep : string;


  signal clk_sig : std_logic;
  signal reset_sig : std_logic;
  signal fifo_dout_sig : std_logic_vector(64-1 downto 0) := (others => '0');
  signal fifo_we_sig : std_logic := '0';
  signal fifo_wclk_sig : std_logic;
  signal fifo_length_sig : std_logic_vector(32-1 downto 0);
  signal fifo_full_sig : std_logic;
  signal S_AXI_ARADDR_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal S_AXI_ARLEN_sig : std_logic_vector(8-1 downto 0) := (others => '0');
  signal S_AXI_ARVALID_sig : std_logic := '0';
  signal S_AXI_ARREADY_sig : std_logic;
  signal S_AXI_ARSIZE_sig : std_logic_vector(3-1 downto 0);
  signal S_AXI_ARBURST_sig : std_logic_vector(2-1 downto 0);
  signal S_AXI_ARCACHE_sig : std_logic_vector(4-1 downto 0);
  signal S_AXI_ARPROT_sig : std_logic_vector(3-1 downto 0);
  signal S_AXI_RDATA_sig : std_logic_vector(64-1 downto 0);
  signal S_AXI_RRESP_sig : std_logic_vector(2-1 downto 0);
  signal S_AXI_RLAST_sig : std_logic;
  signal S_AXI_RVALID_sig : std_logic;
  signal S_AXI_RREADY_sig : std_logic := '0';
  signal request_sig : std_logic;
  signal busy_sig : std_logic := '0';
  signal addr_sig : std_logic_vector(32-1 downto 0);
  signal len_sig : std_logic_vector(8-1 downto 0);

  signal tmp_0001 : std_logic;
  signal tmp_0002 : std_logic;
  type Type_main is (
    main_IDLE,
    main_init,
    main_read0,
    main_read  
  );
  signal main : Type_main := main_IDLE;
  signal main_delay : signed(32-1 downto 0) := (others => '0');
  signal tmp_0003 : std_logic;
  signal tmp_0004 : signed(8-1 downto 0);
  signal read_length : signed(8-1 downto 0) := (others => '0');
  signal tmp_0005 : std_logic;
  signal tmp_0006 : signed(8-1 downto 0);
  signal tmp_0007 : signed(8-1 downto 0);
  signal tmp_0008 : signed(8-1 downto 0);
  signal tmp_0009 : signed(8-1 downto 0);
  signal tmp_0010 : std_logic;
  signal tmp_0011 : std_logic;
  signal tmp_0012 : std_logic;
  signal tmp_0013 : std_logic;
  signal tmp_0014 : std_logic;
  signal tmp_0015 : std_logic;
  signal tmp_0016 : std_logic;
  signal tmp_0017 : std_logic;
  signal tmp_0018 : std_logic;

begin

  clk_sig <= clk;
  reset_sig <= reset;
  fifo_dout <= fifo_dout_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        fifo_dout_sig <= (others => '0');
      else
        if main = main_read0 then
          fifo_dout_sig <= S_AXI_RDATA_sig;
        elsif main = main_read then
          fifo_dout_sig <= S_AXI_RDATA_sig;
        end if;
      end if;
    end if;
  end process;

  fifo_we <= fifo_we_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        fifo_we_sig <= '0';
      else
        if main = main_IDLE then
          fifo_we_sig <= '0';
        elsif main = main_read0 then
          fifo_we_sig <= tmp_0002;
        elsif main = main_read then
          fifo_we_sig <= tmp_0002;
        end if;
      end if;
    end if;
  end process;

  fifo_wclk <= fifo_wclk_sig;
  fifo_wclk_sig <= clk_sig;

  fifo_length_sig <= fifo_length;
  fifo_full_sig <= fifo_full;
  S_AXI_ARADDR <= S_AXI_ARADDR_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        S_AXI_ARADDR_sig <= (others => '0');
      else
        if main = main_init then
          S_AXI_ARADDR_sig <= addr_sig;
        end if;
      end if;
    end if;
  end process;

  S_AXI_ARLEN <= S_AXI_ARLEN_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        S_AXI_ARLEN_sig <= (others => '0');
      else
        if main = main_init then
          S_AXI_ARLEN_sig <= std_logic_vector(tmp_0004);
        end if;
      end if;
    end if;
  end process;

  S_AXI_ARVALID <= S_AXI_ARVALID_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        S_AXI_ARVALID_sig <= '0';
      else
        if main = main_init then
          S_AXI_ARVALID_sig <= '1';
        elsif main = main_read0 then
          S_AXI_ARVALID_sig <= tmp_0005;
        end if;
      end if;
    end if;
  end process;

  S_AXI_ARREADY_sig <= S_AXI_ARREADY;
  S_AXI_ARSIZE <= S_AXI_ARSIZE_sig;
  S_AXI_ARSIZE_sig <= "011";

  S_AXI_ARBURST <= S_AXI_ARBURST_sig;
  S_AXI_ARBURST_sig <= "01";

  S_AXI_ARCACHE <= S_AXI_ARCACHE_sig;
  S_AXI_ARCACHE_sig <= X"3";

  S_AXI_ARPROT <= S_AXI_ARPROT_sig;
  S_AXI_ARPROT_sig <= "000";

  S_AXI_RDATA_sig <= S_AXI_RDATA;
  S_AXI_RRESP_sig <= S_AXI_RRESP;
  S_AXI_RLAST_sig <= S_AXI_RLAST;
  S_AXI_RVALID_sig <= S_AXI_RVALID;
  S_AXI_RREADY <= S_AXI_RREADY_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        S_AXI_RREADY_sig <= '0';
      else
        if main = main_IDLE then
          S_AXI_RREADY_sig <= '1';
        end if;
      end if;
    end if;
  end process;

  request_sig <= request;
  busy <= busy_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        busy_sig <= '0';
      else
        if main = main_IDLE then
          busy_sig <= tmp_0003;
        end if;
      end if;
    end if;
  end process;

  addr_sig <= addr;
  len_sig <= len;

  -- expressions
  tmp_0001 <= '1' when S_AXI_ARREADY_sig = '1' else '0';
  tmp_0002 <= '1' when S_AXI_RVALID_sig = '1' else '0';
  tmp_0003 <= '1' when request_sig = '1' else '0';
  tmp_0004 <= signed(len_sig) - 1;
  tmp_0005 <= '0' when tmp_0001 = '1' else '1';
  tmp_0006 <= read_length - 1;
  tmp_0007 <= tmp_0006 when tmp_0002 = '1' else read_length;
  tmp_0008 <= read_length - 1;
  tmp_0009 <= tmp_0008 when tmp_0002 = '1' else read_length;
  tmp_0010 <= '1' when fifo_full_sig = '0' else '0';
  tmp_0011 <= '1' when S_AXI_RLAST_sig = '1' else '0';
  tmp_0012 <= tmp_0002 and tmp_0011;
  tmp_0013 <= '1' when tmp_0012 = '1' else '0';
  tmp_0014 <= tmp_0002 and tmp_0011;
  tmp_0015 <= '1' when tmp_0014 = '1' else '0';
  tmp_0016 <= '1' when read_length = 1 else '0';
  tmp_0017 <= tmp_0002 and tmp_0016;
  tmp_0018 <= '1' when tmp_0017 = '1' else '0';

  -- sequencers
  process (clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        main <= main_IDLE;
        main_delay <= (others => '0');
      else
        case (main) is
          when main_IDLE => 
            if tmp_0003 = '1' then
              main <= main_init;
            end if;
          when main_init => 
            if tmp_0010 = '1' then
              main <= main_read0;
            end if;
          when main_read0 => 
            if tmp_0001 = '1' then
              main <= main_read;
            end if;
          when main_read => 
            if tmp_0013 = '1' then
              main <= main_IDLE;
            elsif tmp_0015 = '1' then
              main <= main_IDLE;
            elsif tmp_0018 = '1' then
              main <= main_IDLE;
            end if;
          when others => null;
        end case;
      end if;
    end if;
  end process;


  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        read_length <= (others => '0');
      else
        if main = main_init then
          read_length <= signed(len_sig);
        elsif main = main_read0 then
          read_length <= tmp_0007;
        elsif main = main_read then
          read_length <= tmp_0009;
        end if;
      end if;
    end if;
  end process;



end RTL;
