library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity axi_reader_buffer_32 is
  generic (
    BUF_WIDTH : integer := 32
  );
  port (
    clk : in std_logic;
    reset : in std_logic;
    mem_dout : out std_logic_vector(32-1 downto 0);
    mem_we : out std_logic;
    mem_address : out std_logic_vector(32-1 downto 0);
    mem_wclk : out std_logic;
    S_AXI_ARADDR : out std_logic_vector(32-1 downto 0);
    S_AXI_ARLEN : out std_logic_vector(8-1 downto 0);
    S_AXI_ARVALID : out std_logic;
    S_AXI_ARREADY : in std_logic;
    S_AXI_ARSIZE : out std_logic_vector(3-1 downto 0);
    S_AXI_ARBURST : out std_logic_vector(2-1 downto 0);
    S_AXI_ARCACHE : out std_logic_vector(4-1 downto 0);
    S_AXI_ARPROT : out std_logic_vector(3-1 downto 0);
    S_AXI_RDATA : in std_logic_vector(32-1 downto 0);
    S_AXI_RRESP : in std_logic_vector(2-1 downto 0);
    S_AXI_RLAST : in std_logic;
    S_AXI_RVALID : in std_logic;
    S_AXI_RREADY : out std_logic;
    request : in std_logic;
    busy : out std_logic;
    addr : in std_logic_vector(32-1 downto 0);
    len : in std_logic_vector(32-1 downto 0)
  );
end axi_reader_buffer_32;

architecture RTL of axi_reader_buffer_32 is

  attribute mark_debug : string;
  attribute keep : string;


  signal clk_sig : std_logic;
  signal reset_sig : std_logic;
  signal mem_dout_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal mem_we_sig : std_logic := '0';
  signal mem_address_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal mem_wclk_sig : std_logic;
  signal S_AXI_ARADDR_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal S_AXI_ARLEN_sig : std_logic_vector(8-1 downto 0) := (others => '0');
  signal S_AXI_ARVALID_sig : std_logic := '0';
  signal S_AXI_ARREADY_sig : std_logic;
  signal S_AXI_ARSIZE_sig : std_logic_vector(3-1 downto 0);
  signal S_AXI_ARBURST_sig : std_logic_vector(2-1 downto 0);
  signal S_AXI_ARCACHE_sig : std_logic_vector(4-1 downto 0);
  signal S_AXI_ARPROT_sig : std_logic_vector(3-1 downto 0);
  signal S_AXI_RDATA_sig : std_logic_vector(32-1 downto 0);
  signal S_AXI_RRESP_sig : std_logic_vector(2-1 downto 0);
  signal S_AXI_RLAST_sig : std_logic;
  signal S_AXI_RVALID_sig : std_logic;
  signal S_AXI_RREADY_sig : std_logic := '0';
  signal request_sig : std_logic;
  signal busy_sig : std_logic := '0';
  signal addr_sig : std_logic_vector(32-1 downto 0);
  signal len_sig : std_logic_vector(32-1 downto 0);

  signal next_write_addr : signed(32-1 downto 0) := (others => '0');
  signal tmp_0001 : std_logic;
  signal tmp_0002 : std_logic;
  type Type_main is (
    main_IDLE,
    main_init,
    main_read0,
    main_read  
  );
  signal main : Type_main := main_IDLE;
  signal main_delay : signed(32-1 downto 0) := (others => '0');
  signal tmp_0003 : std_logic;
  signal tmp_0004 : signed(32-1 downto 0);
  signal tmp_0005 : signed(8-1 downto 0);
  signal read_length : signed(32-1 downto 0) := (others => '0');
  signal tmp_0006 : std_logic;
  signal tmp_0007 : signed(32-1 downto 0);
  signal tmp_0008 : signed(32-1 downto 0);
  signal tmp_0009 : signed(32-1 downto 0);
  signal tmp_0010 : signed(32-1 downto 0);
  signal tmp_0011 : signed(32-1 downto 0);
  signal tmp_0012 : signed(32-1 downto 0);
  signal tmp_0013 : std_logic;
  signal tmp_0014 : std_logic;
  signal tmp_0015 : std_logic;
  signal tmp_0016 : std_logic;
  signal tmp_0017 : std_logic;
  signal tmp_0018 : std_logic;
  signal tmp_0019 : std_logic;
  signal tmp_0020 : std_logic;

begin

  clk_sig <= clk;
  reset_sig <= reset;
  mem_dout <= mem_dout_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        mem_dout_sig <= (others => '0');
      else
        if main = main_read0 then
          mem_dout_sig <= S_AXI_RDATA_sig;
        elsif main = main_read then
          mem_dout_sig <= S_AXI_RDATA_sig;
        end if;
      end if;
    end if;
  end process;

  mem_we <= mem_we_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        mem_we_sig <= '0';
      else
        if main = main_read0 then
          mem_we_sig <= tmp_0002;
        elsif main = main_read then
          mem_we_sig <= tmp_0002;
        else
          mem_we_sig <= '0';
        end if;
      end if;
    end if;
  end process;

  mem_address <= mem_address_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        mem_address_sig <= (others => '0');
      else
        if main = main_read0 then
          mem_address_sig <= (others => '0');
        elsif main = main_read then
          mem_address_sig <= std_logic_vector(next_write_addr);
        end if;
      end if;
    end if;
  end process;

  mem_wclk <= mem_wclk_sig;
  mem_wclk_sig <= clk_sig;

  S_AXI_ARADDR <= S_AXI_ARADDR_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        S_AXI_ARADDR_sig <= (others => '0');
      else
        if main = main_init then
          S_AXI_ARADDR_sig <= addr_sig;
        end if;
      end if;
    end if;
  end process;

  S_AXI_ARLEN <= S_AXI_ARLEN_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        S_AXI_ARLEN_sig <= (others => '0');
      else
        if main = main_init then
          S_AXI_ARLEN_sig <= std_logic_vector(tmp_0005);
        end if;
      end if;
    end if;
  end process;

  S_AXI_ARVALID <= S_AXI_ARVALID_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        S_AXI_ARVALID_sig <= '0';
      else
        if main = main_init then
          S_AXI_ARVALID_sig <= '1';
        elsif main = main_read0 then
          S_AXI_ARVALID_sig <= tmp_0006;
        end if;
      end if;
    end if;
  end process;

  S_AXI_ARREADY_sig <= S_AXI_ARREADY;
  S_AXI_ARSIZE <= S_AXI_ARSIZE_sig;
  S_AXI_ARSIZE_sig <= "010";

  S_AXI_ARBURST <= S_AXI_ARBURST_sig;
  S_AXI_ARBURST_sig <= "01";

  S_AXI_ARCACHE <= S_AXI_ARCACHE_sig;
  S_AXI_ARCACHE_sig <= X"3";

  S_AXI_ARPROT <= S_AXI_ARPROT_sig;
  S_AXI_ARPROT_sig <= "000";

  S_AXI_RDATA_sig <= S_AXI_RDATA;
  S_AXI_RRESP_sig <= S_AXI_RRESP;
  S_AXI_RLAST_sig <= S_AXI_RLAST;
  S_AXI_RVALID_sig <= S_AXI_RVALID;
  S_AXI_RREADY <= S_AXI_RREADY_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        S_AXI_RREADY_sig <= '0';
      else
        if main = main_IDLE then
          S_AXI_RREADY_sig <= '1';
        end if;
      end if;
    end if;
  end process;

  request_sig <= request;
  busy <= busy_sig;
  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        busy_sig <= '0';
      else
        if main = main_IDLE then
          busy_sig <= tmp_0003;
        end if;
      end if;
    end if;
  end process;

  addr_sig <= addr;
  len_sig <= len;

  -- expressions
  tmp_0001 <= '1' when S_AXI_ARREADY_sig = '1' else '0';
  tmp_0002 <= '1' when S_AXI_RVALID_sig = '1' else '0';
  tmp_0003 <= '1' when request_sig = '1' else '0';
  tmp_0004 <= signed(len_sig) - 1;
  tmp_0005 <= tmp_0004(32 - 24 - 1 downto 0);
  tmp_0006 <= '0' when tmp_0001 = '1' else '1';
  tmp_0007 <= read_length - 1;
  tmp_0008 <= tmp_0007 when tmp_0002 = '1' else read_length;
  tmp_0009 <= next_write_addr + 1;
  tmp_0010 <= read_length - 1;
  tmp_0011 <= tmp_0010 when tmp_0002 = '1' else read_length;
  tmp_0012 <= next_write_addr + 1;
  tmp_0013 <= '1' when S_AXI_RLAST_sig = '1' else '0';
  tmp_0014 <= tmp_0002 and tmp_0013;
  tmp_0015 <= '1' when tmp_0014 = '1' else '0';
  tmp_0016 <= tmp_0002 and tmp_0013;
  tmp_0017 <= '1' when tmp_0016 = '1' else '0';
  tmp_0018 <= '1' when read_length = 1 else '0';
  tmp_0019 <= tmp_0002 and tmp_0018;
  tmp_0020 <= '1' when tmp_0019 = '1' else '0';

  -- sequencers
  process (clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        main <= main_IDLE;
        main_delay <= (others => '0');
      else
        case (main) is
          when main_IDLE => 
            if tmp_0003 = '1' then
              main <= main_init;
            end if;
          when main_init => 
            main <= main_read0;
          when main_read0 => 
            if tmp_0001 = '1' then
              main <= main_read;
            end if;
          when main_read => 
            if tmp_0015 = '1' then
              main <= main_IDLE;
            elsif tmp_0017 = '1' then
              main <= main_IDLE;
            elsif tmp_0020 = '1' then
              main <= main_IDLE;
            end if;
          when others => null;
        end case;
      end if;
    end if;
  end process;


  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        next_write_addr <= (others => '0');
      else
        if main = main_IDLE then
          next_write_addr <= 0;
        elsif main = main_read0 then
          next_write_addr <= tmp_0009;
        elsif main = main_read then
          next_write_addr <= tmp_0012;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if clk'event and clk = '1' then
      if reset = '1' then
        read_length <= (others => '0');
      else
        if main = main_init then
          read_length <= signed(len_sig);
        elsif main = main_read0 then
          read_length <= tmp_0008;
        elsif main = main_read then
          read_length <= tmp_0011;
        end if;
      end if;
    end if;
  end process;



end RTL;
